var generators = require('yeoman-generator');
var chalk      = require('chalk');
var fs         = require('fs');
var path       = require('path');

var routerDependencies = {
  'No routing':   [],
  'React Router': ['react-router', 'redux-simple-router'],
  'Router5':      ['router5', 'redux-router5', 'react-router5', 'router5-history', 'router5-listeners']
}

module.exports = generators.Base.extend({
  initializing: function() {
    this.log(chalk.bold.green('--------------------------------------------------------------------------------'));
    this.log(chalk.bold.green('|                        Welcome to JetReact starter-kit                       |'));
    this.log(chalk.bold.green('--------------------------------------------------------------------------------\n'));
    this.log(chalk.bold.yellow('This generator will walk you through installing a Babel, React, Redux, Webpack application.\n'));
  },
  configuring: function() {
    // Generate our package.json. Make sure to also include the required dependencies for styles
    var scripts = {
      "dev": "webpack-dev-server --config webpack/webpack.development.config.js --hot --colors --progress --inline",
      "build:production": "NODE_ENV=production webpack --config webpack/webpack.production.config.js",
      "build:staging": "webpack --config webpack/webpack.staging.config.js",
      "analyze:production": "NODE_ENV=production webpack --config webpack/webpack.production.config.js --json | analyze-bundle-size",
      "analyze:staging": "webpack --config webpack/webpack.staging.config.js --json | analyze-bundle-size",
      "test": "APP_ENV=test NODE_PATH=./js babel-node test/index.js | tap-spec",
      "deploy:production": "echo 'to be defined...'",
      "deploy:staging": "echo 'to be defined...'",
      "lint": "echo 'to be defined...'"
    };

    var packageSettings = {
      name: this.appName,
      private: true,
      version: '0.1.0',
      description: 'Generated by generator-jetreact',
      main: 'main.js',
      scripts: scripts,
      repository: '',
      keywords: [],
      author: 'Alex Khrustalev'
    };

    this.fs.writeJSON(this.destinationPath('package.json'), packageSettings);
  },
  prompting: function() {
    var done = this.async();

    var prompts = [
      {
        type: 'input',
        name: 'name',
        message: 'Your project name',
        default: this.appname
      },
      // {
      //   type: 'list',
      //   name: 'router',
      //   message: 'Which Router do you want to use?',
      //   choices: [
      //     'No routing',
      //     'React Router',
      //     'Router5'
      //   ]
      // },
      {
        type: 'list',
        name: 'flexbox',
        message: 'Do you want to use Flexbox?',
        choices: [
          'Yes',
          'No'
        ]
      },
      {
        type: 'list',
        name: 'bootstrap',
        message: 'Do you want to use Twitter Bootstrap?',
        choices: [
          'Yes',
          'No'
        ]
      },
      {
        type: 'list',
        name: 'time',
        message: 'Which time/date library would you like to use?',
        choices: [
          'moment',
          'fecha'
        ]
      }
    ];

    this.prompt(prompts, function(answers) {
      this.props = answers;
      done();
    }.bind(this));
  },
  writing: function() {
    var baseTemplatePath = this.templatePath('react-redux-webpack-babel-skeleton');

    // Get all files in our repo and copy the ones we should
    fs.readdir(baseTemplatePath, (err, items) => {
      for(var item of items) {

        var fullPath = path.join(baseTemplatePath, item);

        // Copy all items to our root
        if(fs.lstatSync(fullPath).isDirectory()) {
          this.bulkDirectory(path.join('react-redux-webpack-babel-skeleton', item), item);
        } else {
          this.copy(path.join('react-redux-webpack-babel-skeleton', item), item);
        }
      }
    });
  },
  install: function() {
    this.log(chalk.bold.yellow('Installing dependencies...\n'));
    var dependencies = [
      'ramda',
      'react',
      'react-dom',
      'redux',
      'redux-saga',
      'redux-thunk',
      'redux-form',
      'redux-actions',
      'redux-logger',
      'react-redux',
      'isomorphic-fetch',
      'normalizr',
      'classnames',
      'cuid',
      'path-parser',
      'humps',
      'jquery-param',
      'pluralize',
      'repoint',
      'node-localstorage',
      'react-css-modules'
    ];

    var cssDependencies = [];

    if (this.props.bootstrap === 'Yes') {
      cssDependencies.push('bootstrap')
    }

    if (this.props.flexbox === 'Yes') {
      cssDependencies.push('flexboxgrid')
    }

    dependencies = dependencies.concat([this.props.time])
                               .concat(cssDependencies)
                               .concat(routerDependencies['Router5']);

    this.npmInstall(dependencies, { 'save': true });

    var devDependencies = [
      'webpack',
      'webpack-dev-server',
      'babel-plugin-transform-es2015-destructuring',
      'babel-plugin-transform-object-rest-spread',
      'babel-core',
      'babel-loader',
      'babel-preset-es2015',
      'babel-preset-react',
      'postcss-loader',
      'autoprefixer',
      'style-loader',
      'css-loader',
      'url-loader',
      'file-loader',
      'worker-loader',
      'expose-loader',
      'assets-webpack-plugin',
      'clean-webpack-plugin',
      'babel-polyfill',
      'eslint@^2.9.0',
      'eslint-loader',
      'eslint-plugin-react',
      'eslint-config-airbnb',
      'eslint-plugin-import',
      'eslint-plugin-jsx-a11y@^1.2.0'
    ];

    var testDependencies = [
      'tape',
      'tap-spec',
      'enzyme',
      'sinon'
    ];

    devDependencies = devDependencies.concat(testDependencies);

    this.npmInstall(devDependencies, { 'saveDev': true });
  }
});
